-----------------------------------------------------------------------------------------------
-- Client Lua Script for WADS (WildStar Annoyance Detection System)
-- Copyright (c) NCsoft. All rights reserved

-- Author: Grimmblade
-- Server: Orias
-- Curse Username: Knightlife
-----------------------------------------------------------------------------------------------
require "Window"
 
-----------------------------------------------------------------------------------------------
-- WADS Module Definition
-----------------------------------------------------------------------------------------------
local WADS = {} 

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- No global constants at this time

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function WADS:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self:Reset()

    return o
end

function WADS:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- OnLoad
-----------------------------------------------------------------------------------------------
function WADS:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("WADS.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	-- hook into chat
	Apollo.GetPackage("Gemini:Hook-1.0").tPackage:Embed(self)
	self:RawHook(Apollo.GetAddon("ChatLog"),"OnChatMessage")
end

-----------------------------------------------------------------------------------------------
-- OnDocLoaded
-----------------------------------------------------------------------------------------------
function WADS:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "WADSForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("wads", "OnSlashCommand", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- Persistence handling
-----------------------------------------------------------------------------------------------
function WADS:OnSave(eLevel)
	-- TODO implement option to save accountwide or per character
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	return
	{
		bActive = self.bActive,
		tStrings = self.tStrings,
		nVerbosity = self.nVerbosity
	}
end

function WADS:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	if tData ~= nil then
		self.bActive = tData.bActive
		self.tStrings = tData.tStrings
		self.nVerbosity = tData.nVerbosity
	end
end

-----------------------------------------------------------------------------------------------
-- Reset all variables in addon to defaults
-----------------------------------------------------------------------------------------------
function WADS:Reset()
	self.bActive = true
	self.tStrings = { }
	self.nVerbosity = 1
end

-----------------------------------------------------------------------------------------------
-- List all spam filters
-----------------------------------------------------------------------------------------------
function WADS:ListIgnore()
	Print("WADS filters:")

	for i=1, table.getn(self.tStrings) do
		Print("# " .. i .. " - " .. self.tStrings[i])
	end
end

-----------------------------------------------------------------------------------------------
-- Add a spam filter
-----------------------------------------------------------------------------------------------
function WADS:AddLink(string)
	if string ~= nil and string ~= "" then
		string = self:EscapeSpecialChars(string)
		
		local found = 0
		for i=1, table.getn(self.tStrings) do
		  	if self.tStrings[i] == string then
				Print("WADS: " .. string .. " already in the list")
				found = 1
			end
        end

		if found == 0 then
		    table.insert(self.tStrings, string)
			Print("WADS: " .. string.. " added to the list")
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Slash command handling
-----------------------------------------------------------------------------------------------
function WADS:OnSlashCommand(strCommand, strArg)
	-- Shows a UI window, doesn't do anything yet so not doing it
	--self.wndMain:Invoke() -- show the window
	
	msg = string.lower(strArg)

	if (msg == "" or msg == nil) then
		Print("WADS commands:")
		Print("enable - enables WADS")
		Print("disable - disables WADS")
		Print("add <string> - adds a string to the filter list")
		Print("list - shows the list of filter strings")
		Print("remove <number> - removes a filter from the list with the given number")
		Print("verbosity <number> - 0: silent mode 1: normal mode 2: verbose mode")
		Print("reset - deletes all filters and sets WADS to default settings")
	elseif msg == "enable" then
		self.bActive = true
		Print("WADS activated")
	elseif msg == "disable" then
		self.bActive = false
		Print("WADS deactivated")
	elseif msg == "reset" then
		self:Reset()
		Print("WADS reset to default settings")
	elseif string.find(msg, 'add (.+)') then
		_, _, string = string.find(msg, 'add (.+)')
		self:AddLink(string)
	elseif msg == "list" then
		self:ListIgnore()
	elseif string.find(msg, 'verbosity (.+)') then
		local _, _, vNum = string.find(msg, 'verbosity (.+)')
		vNum = tonumber(vNum)
		if (vNum ~= nil) then
			if vNum == 0 then
				Print("WADS: silent mode enabled")
				self.nVerbosity = 0
			elseif vNum == 1 then
				Print("WADS: normal mode enabled")
				self.nVerbosity = 1
			elseif vNum == 2 then
				Print("WADS: verbose mode enabled")
				self.nVerbosity = 2
			else
				Print("WADS: unknown verbosity, not changing current setting of " .. self.nVerbosity)
			end
		else
			Print("WADS: unknown verbosity, not changing current setting of " .. self.nVerbosity)
		end
	elseif string.find(msg, 'remove (%d+)') then
		local _, _, num = string.find(msg, 'remove (%d+)')
		num = tonumber(num)
		if (num ~= nil) then
			if self.tStrings[num] ~= nil then
				local origname = self.tStrings[num]
				table.remove(self.tStrings, num)
				Print("WADS: string " .. origname .. " removed from the list")
			else
				Print("WADS: invalid string number...no strings removed from the list")
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- UI functions - unused at the moment since the UI isn't functional
-----------------------------------------------------------------------------------------------
function WADS:OnOK()
	self.wndMain:Close() -- hide the window
end

function WADS:OnCancel()
	self.wndMain:Close() -- hide the window
end

-----------------------------------------------------------------------------------------------
-- EscapeSpecialChars
-- Some characters don't play well with lua strings / string finding
-- This function escapes them so string.find() doesn't freak out / they get read from the table correctly
-----------------------------------------------------------------------------------------------
function WADS:EscapeSpecialChars(s)
	return (s:gsub('[%-%.%+%[%]%(%)%$%^%%%?%*]','%%%1'):gsub('%z','%%z'))
end

-----------------------------------------------------------------------------------------------
-- OnChatMessage
-----------------------------------------------------------------------------------------------
-- Save these off because you can receive the same message multiple times in Carbine's UI (one for each channel at least)
-- This way when the text is blocked you only see the output once
-- Also makes it so if someone spams the same thing over and over you only see the output from WADS once (if output is enabled)
local lastSender, lastFilter

function WADS:OnChatMessage(luaCaller, channelCurrent, tMessage)
	local author = tMessage.strSender
	
	-- As far as I understand, this should contain the whole message...will need to update this if that's not the case, but the
	-- documentation on the chat UI isn't useful at the moment / some variables Carbine had in their documentation don't exist at all...
	local msg = tMessage.arMessageSegments[1].strText
	
	if (self.bActive == true and channelCurrent:GetName() ~= nil and channelCurrent:GetName() ~= "Debug") then
		msg = string.lower(msg)
		msg = self:EscapeSpecialChars(msg)
		local bFound = false

		for i=1, table.getn(self.tStrings) do
			-- plain text string find, lots of guilds / spam sites have crazy characters in them so passing them to find can crash it in non-plain mode
			if string.find(msg, self.tStrings[i], 1, true) then
				bFound = true
				if lastSender ~= author or lastFilter ~= self.tStrings[i] then
					if self.nVerbosity == 0 then
						-- silent mode, don't print anything
					elseif self.nVerbosity == 1 then
						-- Notes from WoW:
						-- Gold farmers are spamming alternating website messages, which causes just as much spam blocking the spam as the spam itself
						-- Blocked filter 1, blocked filter 2, blocked filter 1...
						-- Need to find a way to fix this in Carbine's UI...would help to throw all this in a UI box or at least color the text
						-- Maybe have a stack of previously blocked messages?  That might be the easiest fix if this becomes an issue in WildStar...
						Print("WADS: spam message from "..author.." blocked")
					elseif self.nVerbosity == 2 then
						Print("WADS: message with "..self.tStrings[i].." sent by "..author.." would've been blocked, printing to chat window")
						Print("Full message from "..author..": "..msg)
					end

					lastSender = author
					lastFilter = self.tStrings[i]
				end
				
				break
			end
		end
		
		if not bFound then
			self.hooks[Apollo.GetAddon("ChatLog")].OnChatMessage(luaCaller, channelCurrent, tMessage)
		end
	else
		self.hooks[Apollo.GetAddon("ChatLog")].OnChatMessage(luaCaller, channelCurrent, tMessage)
	end
	
end

-----------------------------------------------------------------------------------------------
-- WADS Instance
-----------------------------------------------------------------------------------------------
local WADSInst = WADS:new()
WADSInst:Init()
